<?php

namespace Drupal\countdown_clock\Form;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines the settings form for the Countdown Clock.
 */
class CountdownSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'countdown_clock_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['countdown_clock.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('countdown_clock.settings');

    // Attach the countdown JavaScript library.
    $form['#attached']['library'][] = 'countdown_clock/countdown_script';
    $form['#attached']['library'][] = 'countdown_clock/countdown_clock';

    // Countdown title field.
    $form['countdown_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Countdown Title'),
      '#description' => $this->t('Enter a title for the countdown.'),
      '#default_value' => $config->get('countdown_title') ?: '',
      '#required' => TRUE,
    ];

    // Fetch the end date from configuration.
    $end_date = $config->get('end_date');
    $end_date_value = $end_date ? DrupalDateTime::createFromTimestamp($end_date) : NULL;

    // Date and time field for countdown end date.
    $form['end_date'] = [
      '#type' => 'datetime',
      '#title' => $this->t('End Date'),
      '#description' => $this->t('Select the date and time when the countdown should end.'),
      '#default_value' => $end_date_value,
      '#required' => TRUE,
    ];

    // Show/Hide functionality for countdown parts.
    $form['show_parts'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Show/Hide Countdown Parts'),
      '#description' => $this->t('Select which parts of the countdown timer should be displayed.'),
      '#required' => TRUE,
      '#options' => [
        'show_days' => $this->t('Show Days'),
        'show_hours' => $this->t('Show Hours'),
        'show_minutes' => $this->t('Show Minutes'),
        'show_seconds' => $this->t('Show Seconds'),
      ],
      '#default_value' => [
        'show_days' => $config->get('show_days') ? 'show_days' : NULL,
        'show_hours' => $config->get('show_hours') ? 'show_hours' : NULL,
        'show_minutes' => $config->get('show_minutes') ? 'show_minutes' : NULL,
        'show_seconds' => $config->get('show_seconds') ? 'show_seconds' : NULL,
      ],
    ];

    // Finished message field.
    $form['finished_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Finished Message'),
      '#default_value' => $config->get('finished_message') ?: $this->t('Countdown finished!'),
      '#description' => $this->t('Enter the message to display when the countdown ends.'),
      '#required' => TRUE,
    ];

    // Color settings fieldset.
    $form['color_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Color Settings'),
    ];

    $form['color_settings']['background_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Background Color'),
      '#default_value' => $config->get('background_color') ?: '#ffffff',
    ];

    $form['color_settings']['text_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Text Color'),
      '#default_value' => $config->get('text_color') ?: '#000000',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $end_date = $form_state->getValue('end_date');
    $end_date_timestamp = $end_date ? $end_date->getTimestamp() : NULL;

    $this->config('countdown_clock.settings')
      ->set('countdown_title', $form_state->getValue('countdown_title'))
      ->set('end_date', $end_date_timestamp)
      ->set('finished_message', $form_state->getValue('finished_message'))
      ->set('background_color', $form_state->getValue('background_color'))
      ->set('text_color', $form_state->getValue('text_color'))
      ->set('show_days', $form_state->getValue(['show_parts', 'show_days']))
      ->set('show_hours', $form_state->getValue(['show_parts', 'show_hours']))
      ->set('show_minutes', $form_state->getValue(['show_parts', 'show_minutes']))
      ->set('show_seconds', $form_state->getValue(['show_parts', 'show_seconds']))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
