<?php

namespace Drupal\countdown_clock\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'Countdown Block' block.
 *
 * @Block(
 *   id = "countdown_block",
 *   admin_label = @Translation("Countdown Block"),
 *   category = @Translation("Countdown Block"),
 * )
 */
class CountdownBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Load configuration.
    $config = \Drupal::config('countdown_clock.settings');

    // Fetch the end date and convert it to a timestamp.
    $end_date = $config->get('end_date');
    $end_date_value = $end_date ? DrupalDateTime::createFromTimestamp($end_date) : NULL;
    $end_timestamp = $end_date_value ? $end_date_value->getTimestamp() : 0;

    // Set background and text color.
    $background_color = $config->get('background_color') ?: '#ffffff';
    $text_color = $config->get('text_color') ?: '#000000';

    // Fetch show parts configuration.
    $show_days = $config->get('show_days') ? TRUE : FALSE;
    $show_hours = $config->get('show_hours') ? TRUE : FALSE;
    $show_minutes = $config->get('show_minutes') ? TRUE : FALSE;
    $show_seconds = $config->get('show_seconds') ? TRUE : FALSE;

    // Retrieve the countdown title from the configuration.
    $countdown_title = $config->get('countdown_title') ?: '';

    // Retrieve the event URL from the block configuration.
    $block_config = $this->getConfiguration();
    $event_url = $block_config['event_url'] ?? '';

    // Build the output.
    $output = [
      '#type' => 'inline_template',
      '#template' => '
        <style>
          #countdown-timer .countdown-number span {
            color: {{ text_color }};
            background-color: {{ background_color }};
          }
        </style>
        <div id="countdown-timer">
          {% if countdown_title %}
            <h5>{{ countdown_title }}</h5>
          {% endif %}
            <div class="countdown-timer">
            {% if show_days %}
              <div class="countdown-days">
                <div class="countdown-number" id="days"><span>0</span><span>0</span></div>
                <div class="countdown-label">Days</div>
              </div>
            {% endif %}
            {% if show_hours %}
              <div class="countdown-hours">
                <div class="countdown-number" id="hours"><span>0</span><span>0</span></div>
                <div class="countdown-label">Hours</div>
              </div>
            {% endif %}
            {% if show_minutes %}
              <div class="countdown-minutes">
                <div class="countdown-number" id="minutes"><span>0</span><span>0</span></div>
                <div class="countdown-label">Minutes</div>
              </div>
            {% endif %}
            {% if show_seconds %}
              <div class="countdown-seconds">
                <div class="countdown-number" id="seconds"><span>0</span><span>0</span></div>
                <div class="countdown-label">Seconds</div>
              </div>
            {% endif %}
          </div>
        </div>
        {% if event_url %}
          <div class="event-url">
            <a href="{{ event_url }}" target="_blank">{{ "Click here for event details" }}</a>
          </div>
        {% endif %}
      ',
      '#context' => [
        'text_color' => $text_color,
        'background_color' => $background_color,
        'countdown_title' => $countdown_title,
        'show_days' => $show_days,
        'show_hours' => $show_hours,
        'show_minutes' => $show_minutes,
        'show_seconds' => $show_seconds,
        'event_url' => $event_url,
      ],
      '#attached' => [
        'library' => [
          'countdown_clock/countdown_script',
      // Ensure this library is attached.
          'countdown_clock/countdown_clock',
        ],
        'drupalSettings' => [
          'countdownClock' => [
            'endTimestamp' => $end_timestamp,
          ],
        ],
      ],
    ];

    // Ensure the CSS is also attached.
    $output['#attached']['library'][] = 'countdown_clock/countdown_clock';

    return $output;
  }

  /**
   * Implements a block form handler.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    // Retrieve existing configuration for this block.
    $config = $this->getConfiguration();

    // Event URL field.
    $form['event_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event URL'),
      '#description' => $this->t('Provide the URL for the event.'),
      '#default_value' => $config['event_url'] ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Save the event URL in block configuration.
    $this->setConfigurationValue('event_url', $form_state->getValue('event_url'));
  }

}
